#include "file_output_base.h"

#include "e32std.h"

const TLitC<3> Mfile_output_base::nl={2, L"\r\n"};

EXPORT_C void Mfile_output_base::write_to_output(const TDesC8& str)
{
	if (NoSpaceLeft()) return;
	file.Write(str);
}

EXPORT_C void Mfile_output_base::write_nl()
{
	write_to_output(nl);
}

EXPORT_C void Mfile_output_base::write_to_output(const TDesC& str)
{
	if (NoSpaceLeft()) return;
	CC()->ConvertFromUnicode(buf, str);
	file.Write(buf);
}

EXPORT_C void Mfile_output_base::write_time(const TTime& time)
{
	TBuf<256> buffer;
	TDateTime dt;
	dt=time.DateTime();
	_LIT(KFormatTxt,"%04d%02d%02dT%02d%02d%02d ");
	buffer.Format(KFormatTxt, dt.Year(), (TInt)dt.Month()+1, (TInt)dt.Day()+1,
		dt.Hour(), dt.Minute(), dt.Second());
	write_to_output(buffer);
}	

EXPORT_C void Mfile_output_base::write_time()
{
	TTime time;
	time.HomeTime();
	write_time(time);
}

EXPORT_C void Mfile_output_base::open_file(const TDesC& prefix)
{
	file.Close();
	TBuf<200> filen;
	make_filename(filen, DataDir(), prefix);
	User::LeaveIfError(file.Replace(Fs(), filen, EFileShareAny | EFileStreamText | EFileWrite));
}

EXPORT_C void Mfile_output_base::close_file()
{
	file.Close();
}

EXPORT_C void Mfile_output_base::make_filename(TDes& into_buffer, const TDesC& dir, const TDesC& prefix)
{
	make_filename(into_buffer, dir, prefix, _L("txt"));
}

EXPORT_C void Mfile_output_base::make_filename(TDes& into_buffer, const TDesC& dir, const TDesC& prefix, const TDesC& extension)
{
	TTime time;
	TDateTime dt;
	time.HomeTime();
	dt=time.DateTime();
	_LIT(KFormatTxt,"%S%S-%04d%02d%02dT%02d%02d%02d.%S");
	into_buffer.Format(KFormatTxt, &dir, 
		&prefix, dt.Year(), (TInt)dt.Month()+1, (TInt)dt.Day()+1,
		dt.Hour(), dt.Minute(), dt.Second(), &extension);
}

EXPORT_C void Mfile_output_base::ConstructL(const TDesC& prefix, bool do_open_file)
{
	if (file_prefix) delete file_prefix; file_prefix=0;
	file_prefix=HBufC::NewL(prefix.Length());
	*file_prefix=prefix;
	if (do_open_file) {
		open_file(prefix);
	}

	state=CCnvCharacterSetConverter::KStateDefault;
}


EXPORT_C Mfile_output_base::Mfile_output_base(MApp_context& Context): MContextBase(Context), file_prefix(0) { }

EXPORT_C Mfile_output_base::~Mfile_output_base() 
{
	file.Close();
	delete file_prefix;
}

EXPORT_C Cfile_output_base::Cfile_output_base(MApp_context& Context) : Mfile_output_base(Context)
{
}

EXPORT_C Cfile_output_base::~Cfile_output_base()
{
}

EXPORT_C Cfile_output_base* Cfile_output_base::NewL(MApp_context& Context, const TDesC& prefix, bool do_open_file)
{
	Cfile_output_base* ret;
	ret=new (ELeave) Cfile_output_base(Context);
	CleanupStack::PushL(ret);
	ret->ConstructL(prefix, do_open_file);
	CleanupStack::Pop();
	return ret;
}