#include "presence_data.h"

EXPORT_C void TCellInfo::Persist(CXmlBuf* into)
{
	into->BeginElement(_L("location"));
	into->BeginElement(_L("location.value"));
	into->Leaf(_L("location.network"), iNetwork);
	TBuf<10> val;
	val.AppendNum(iLAC);
	into->Leaf(_L("location.lac"), val);
	val.Zero();
	val.AppendNum(iCellId);
	into->Leaf(_L("location.cellid"), val);
	into->EndElement(_L("location.value"));
	into->EndElement(_L("location"));
}

EXPORT_C void TLocInfo::Persist(CXmlBuf* into)
{
	_LIT(KFormatTxt,"%04d%02d%02dT%02d%02d%02d");
	TBuf<20> buf;

	into->BeginElement(_L("base"));
	TDateTime dt=iCoPUpdated.DateTime();

	buf.Format(KFormatTxt, dt.Year(), (TInt)dt.Month()+1, (TInt)dt.Day()+1,
		dt.Hour(), dt.Minute(), dt.Second());
	if (iFlags & EPrevious) {
		into->Leaf(_L("base.previous.left"), buf);
		into->Leaf(_L("base.previous"), *iCurrentOrPrevious);
	}
	if (iFlags & ECurrent) {
		into->Leaf(_L("base.current.arrived"), buf);
		into->Leaf(_L("base.current"), *iCurrentOrPrevious);
	}
	if (iFlags & ENext) {
		into->Leaf(_L("base.next"), *iNext);
	}
	into->EndElement(_L("base"));
}

EXPORT_C void TProfileInfo::Persist(CXmlBuf* into)
{
	into->BeginElement(_L("profile"));
	into->BeginElement(_L("profile.value"));
	TBuf<10> val;
	val.AppendNum(iProfileId);
	into->Leaf(_L("profile.id"), val);
	into->Leaf(_L("profile.name"), iProfileName);
	val.Zero(); val.AppendNum(iRingingType);
	into->Leaf(_L("profile.ringtype"), val);
	val.Zero(); val.AppendNum(iRingingVolume);
	into->Leaf(_L("profile.ringvolume"), val);
	if (iVibra) {
		into->Leaf(_L("profile.vibrate"), _L("true"));
	} else {
		into->Leaf(_L("profile.vibrate"), _L("false"));
	}
	into->EndElement(_L("profile.value"));
	into->EndElement(_L("profile"));
}

EXPORT_C void TActivityInfo::Persist(CXmlBuf* into)
{
	into->BeginElement(_L("useractivity"));

	if (iMode==EIdle) {
		into->Leaf(_L("useractivity.value"), _L("idle"));
	} else {
		into->Leaf(_L("useractivity.value"), _L("active"));
	}

	into->EndElement(_L("useractivity"));
}

EXPORT_C CPresenceData* CPresenceData::NewL(const TDesC& Xml, const TTime& send_timestamp)
{
	auto_ptr<CPresenceData> ret(new (ELeave) CPresenceData(send_timestamp));
	ret->ConstructL(Xml);
	return ret.release();
}

EXPORT_C CPresenceData::~CPresenceData()
{
	delete iParser;
	delete iParseState;
	delete iXml;
	delete iChars;
}

EXPORT_C const TCellInfo& CPresenceData::CellInfo() const
{
	return iCellInfo;
}

EXPORT_C const TLocInfo& CPresenceData::LocInfo() const
{
	return iLocInfo;
}

EXPORT_C const TActivityInfo& CPresenceData::ActivityInfo() const
{
	return iActivityInfo;
}

EXPORT_C const TProfileInfo& CPresenceData::ProfileInfo() const
{
	return iProfileInfo;
}

EXPORT_C const TDesC& CPresenceData::RawXml() const
{
	return *iXml;
}

EXPORT_C const TTime& CPresenceData::SendTimeStamp() const
{
	return iSendTimeStamp;
}

CPresenceData::CPresenceData(const TTime& send_timestamp) : iSendTimeStamp(send_timestamp)
{
}

void CPresenceData::ConstructL(const TDesC& Xml)
{
	iChars=HBufC::NewL(512);
	iXml=Xml.AllocL();
	iParseState=CList<TParseState>::NewL();
	iParseState->Push(EIdle);
	iParser=CXmlParser::NewL(*this);
	iParser->Parse( (char*)iXml->Ptr(), iXml->Size(), true);
	iParseState->Pop();
	delete iParser; iParser=0;
	delete iParseState; iParseState=0;
}

void CPresenceData::SetStamp(MXmlInfo& Info)
{
	Info.iUpdated=iStamp;
}

void CPresenceData::StartElement(const XML_Char *name,
			const XML_Char**  /*atts*/)
{
	TPtrC elem_name((TUint16 *)name);
	TParseState next=iParseState->Top();
		if (! elem_name.Compare(_L("location") ) ) {
			SetStamp(iCellInfo);
		} else if (! elem_name.Compare(_L("profile") ) ) {
			SetStamp(iProfileInfo);
		} else if (! elem_name.Compare(_L("base") ) ) {
			SetStamp(iLocInfo);
		} else if (! elem_name.Compare(_L("useractivity") ) ) {
			SetStamp(iActivityInfo);
		} else if (! elem_name.Compare(_L("profile.id") ) ) {
			next=EProfileId;
		} else if (! elem_name.Compare(_L("profile.name") ) ) {
			next=EProfileName;
		} else if (! elem_name.Compare(_L("profile.ringtype") ) ) {
			next=EProfileRingType;
		} else if (! elem_name.Compare(_L("profile.ringvolume") ) ) {
			next=EProfileRingVolume;
		} else if (! elem_name.Compare(_L("profile.vibrate") ) ) {
			next=EProfileVibra;
		} else if (! elem_name.Compare(_L("location.network") ) ) {
			next=ECellNw;
		} else if (! elem_name.Compare(_L("location.lac") ) ) {
			next=ECellLAC;
		} else if (! elem_name.Compare(_L("location.cellid") ) ) {
			next=ECellId;
		} else if (! elem_name.Compare(_L("base.previous") ) ) {
			next=ELocPrevious;
		} else if (! elem_name.Compare(_L("base.current") ) ) {
			next=ELocCurrent;
		} else if (! elem_name.Compare(_L("base.previous.left") ) ) {
			next=ELocPoCUpdate;
		} else if (! elem_name.Compare(_L("base.current.arrived") ) ) {
			next=ELocPoCUpdate;
		} else if (! elem_name.Compare(_L("base.next") ) ) {
			next=ELocNext;
		} else if (! elem_name.Compare(_L("useractivity.value") ) ) {
			next=EActivityMode;
		} else if (! elem_name.Compare(_L("datetime") ) ) {
			next=EStamp;
		}
	iChars->Des().Zero();
	iParseState->Push(next);
}

EXPORT_C TTime CPresenceData::ParseTimeL(const TDesC& Str)
{
	TInt year, month, day, hour, min, sec, err;
        TLex lex;
        
	lex=Str.Mid(0, 4);
        err =  lex.Val(year);

        lex=Str.Mid(4, 2);
        err += lex.Val(month);

        lex=Str.Mid(6, 2);
        err += lex.Val(day);

        lex=Str.Mid(9, 2);
        err += lex.Val(hour);

        lex=Str.Mid(11, 2);
        err += lex.Val(min);

        lex=Str.Mid(13, 2);
        err += lex.Val(sec);

	

	if (err ==KErrNone)
	{
		TDateTime dt; TTime Stamp;
		dt.Set(year,TMonth(month-1),day-1,hour,min,sec,0);
		Stamp=TTime(dt);
		return Stamp;
	}
	else
	{
		TTime dummy = TTime(_L("20000101:000000.000000"));
		return dummy;
	}
}

EXPORT_C void CPresenceData::MapLocInfo(HBufC * locInfo)
{
	TPtr tmp = locInfo->Des();
	TInt pos = tmp.FindC(_L("helsinki"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 8, _L("HKI"));
	}
	pos = tmp.FindC(_L("tampere"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 7, _L("TRE"));
	}
	pos = tmp.FindC(_L("espoo"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 5, _L("ESP"));
	}
	pos = tmp.FindC(_L("turku"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 5, _L("TKU"));
	}
	pos = tmp.FindC(_L("oulu"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 4, _L("OUL"));
	}
	pos = tmp.FindC(_L("jyvskyl"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 9, _L("JKY"));
	}
	pos = tmp.FindC(_L("vantaa"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 6, _L("VNT"));
	}
	pos = tmp.FindC(_L("kuopio"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 6, _L("KPO"));
	}
	pos = tmp.FindC(_L("Lahti"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 5, _L("LHT"));
	}
	pos = tmp.FindC(_L("kirkkoniemi"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 11, _L("KNM"));
	}
	pos = tmp.FindC(_L("Joensuu"));
	if (pos != KErrNotFound)
	{
		tmp.Replace(pos, 7 ,_L("JSU"));
	}

	
}

void CPresenceData::ParseStamp(TTime& Stamp)
{
	Stamp=ParseTimeL(*iChars);
	
}

void CPresenceData::EndElement(const XML_Char * /*name*/)
{
	iChars->Des().TrimAll();
	TLex lex(*iChars);

	switch (iParseState->Pop()) {
	case EStamp:
		ParseStamp(iStamp);
		break;
	case ELocPoCUpdate:
		ParseStamp(iLocInfo.iCoPUpdated);
		break;
	case EProfileId:
		lex.Val(iProfileInfo.iProfileId);
		break;
	case EProfileRingType:
		lex.Val(iProfileInfo.iRingingType);
		break;
	case EProfileRingVolume:
		lex.Val(iProfileInfo.iRingingVolume);
		break;
	case EProfileVibra:
		if (! iChars->Compare(_L("true")) ) {
			iProfileInfo.iVibra=ETrue;
		} else {
			iProfileInfo.iVibra=EFalse;
		}
		break;
	case EProfileName:
		iProfileInfo.iProfileName=*iChars;
		break;
	case ECellNw:
		iCellInfo.iNetwork=*iChars;
		break;
	case ECellLAC:
		lex.Val(iCellInfo.iLAC);
		break;
	case ECellId:
		lex.Val(iCellInfo.iCellId);
		break;
	case ELocPrevious:
		delete iLocInfo.iCurrentOrPrevious; iLocInfo.iCurrentOrPrevious=0;
		MapLocInfo(iChars);
		iLocInfo.iCurrentOrPrevious=iChars->AllocL();
		
		iLocInfo.iFlags|=TLocInfo::EPrevious;
		break;

	case ELocCurrent:
		delete iLocInfo.iCurrentOrPrevious; iLocInfo.iCurrentOrPrevious=0;
		MapLocInfo(iChars);
		iLocInfo.iCurrentOrPrevious=iChars->AllocL();
		
		iLocInfo.iFlags|=TLocInfo::ECurrent;
		break;

	case ELocNext:
		delete iLocInfo.iNext; iLocInfo.iNext=0;
		MapLocInfo(iChars);
		iLocInfo.iNext=iChars->AllocL();
		
		iLocInfo.iFlags|=TLocInfo::ENext;
		break;

	case EActivityMode:
		if (! iChars->Compare(_L("idle")) ) {
			iActivityInfo.iMode=TActivityInfo::EIdle;
		} else {
			iActivityInfo.iMode=TActivityInfo::EActive;
		}
		break;
	default:
		break;
	}
}

void CPresenceData::CharacterData(const XML_Char *s,
			    int len)
{
	TPtrC c( (TUint16*)s, len);
	while (c.Length()+iChars->Length() > iChars->Des().MaxLength()) {
		iChars=iChars->ReAllocL(iChars->Des().MaxLength()*2);
	}
	iChars->Des().Append(c);
}

void CPresenceData::Error(XML_Error Code, const XML_LChar * String, long /*ByteIndex*/)
{
	iErrorCode=Code;
	iError=TPtrC( (TUint16*) String).AllocL();
}


