#include "timeout.h"
#include "symbian_auto_ptr.h"

class CTimeOutImpl : public CTimeOut {
public:
	~CTimeOutImpl();
private:
	virtual void Wait(int seconds);
	virtual void Reset();

	CTimeOutImpl(MTimeOut& i_cb);
	void ConstructL();
	void RunL();

	MTimeOut& cb;
	TTime	iWaitUntil;

	friend class CTimeOut;
};

EXPORT_C void MTimeOut::expired(CBase* Source)
{
	expired();
}

CTimeOut::CTimeOut() : CTimer(EPriorityNormal)
{
}

CTimeOut::~CTimeOut()
{
}

CTimeOutImpl::CTimeOutImpl(MTimeOut& i_cb) : cb(i_cb)
{
}

void CTimeOutImpl::ConstructL()
{
	CTimer::ConstructL();
	CActiveScheduler::Add(this);
}

EXPORT_C CTimeOut* CTimeOut::NewL(MTimeOut& i_cb)
{
	auto_ptr<CTimeOutImpl> ret (new (ELeave) CTimeOutImpl(i_cb));
	ret->ConstructL();
	return ret.release();
}

void CTimeOutImpl::RunL()
{
	TTime now; now.HomeTime(); now+=TTimeIntervalMicroSeconds(100*1000);
	if (now < iWaitUntil) {
		At(iWaitUntil);
	} else {
		cb.expired(this);
	}
}

void CTimeOutImpl::Wait(int seconds)
{
	Reset();
	TTime at;
	at.HomeTime();
	at+=TTimeIntervalSeconds(seconds);
	iWaitUntil=at;
	At(at);
}

void CTimeOutImpl::Reset()
{
	Cancel();
}

CTimeOutImpl::~CTimeOutImpl()
{
	Cancel();
}