#include "xml.h"
#include "symbian_auto_ptr.h"

CXmlParser* CXmlParser::NewL(MXmlHandler& Handler)
{
	auto_ptr<CXmlParser> ret(new (ELeave) CXmlParser(Handler));
	ret->ConstructL();
	return ret.release();
}

void CXmlParser::Parse(const char* buf, int len, int isFinal)
{
	if (!XML_Parse(iParser, buf, len, isFinal)) {
		XML_Error code=XML_GetErrorCode(iParser);
		iHandler.Error(code, XML_ErrorString(code), XML_GetCurrentByteIndex(iParser));
	}
}

CXmlParser::~CXmlParser()
{
	XML_ParserFree(iParser);
}

void CXmlParser::StartElement(void *userData,
			   const XML_Char *name,
			   const XML_Char **atts)
{
	MXmlHandler* h=(MXmlHandler*)userData;
	h->StartElement(name, atts);
}

void CXmlParser::EndElement(void *userData,
			 const XML_Char *name)
{
	MXmlHandler* h=(MXmlHandler*)userData;
	h->EndElement(name);
}

void CXmlParser::CharacterData(void *userData,
			    const XML_Char *s,
			    int len)
{
	MXmlHandler* h=(MXmlHandler*)userData;
	h->CharacterData(s, len);
}

CXmlParser::CXmlParser(MXmlHandler& Handler) : iHandler(Handler)
{
}

void CXmlParser::ConstructL()
{
	iParser=XML_ParserCreate(0);
	XML_SetElementHandler(iParser, StartElement, EndElement);
	XML_SetCharacterDataHandler(iParser, CharacterData);
	XML_SetUserData(iParser, &iHandler);
}
