//CContextServer.cpp
#include "ver.h"
#include "ContextServer.h"
#include "ContextServerSession.h"
#include "ContextCommon.h"
#include <E32SVR.H>
#include <basched.h>

#include <flogger.h>

void Log(const TDesC& msg);

CContextServer::CContextServer(TInt aPriority) : CServer(aPriority)
{
	
}

CContextServer::~CContextServer()
{
	delete iPresenceInfo;
	iPresenceInfo = NULL;

//	delete iConnectionOpener;
//	iConnectionOpener = NULL;

	delete iJabberClient;
	iJabberClient = NULL;


	iSessionIter.SetToFirst();
	for (;;)
        {
		CContextServerSession* session;
		session = reinterpret_cast<CContextServerSession*>(iSessionIter++);
		if (!session)
		{
			break;
		}

		session->~CContextServerSession();
        }
}

CContextServer* CContextServer::NewL()
{
	CContextServer* contextServer = CContextServer::NewLC();
    	CleanupStack::Pop(contextServer);
    	return contextServer;
}

CContextServer* CContextServer::NewLC()
{
	CContextServer* contextServer = new (ELeave) CContextServer(EPriorityNormal);
    	CleanupStack::PushL(contextServer);
    	contextServer->ConstructL();
    	return contextServer;
}

void CContextServer::ConstructL()
{
	iPresenceInfo = CPresenceInfo::NewL();

	iJabberClient = CSocketsEngine::NewL(*this);

	iMessageFromLog = RMessage();
	iMessageFromBook = RMessage();

	StartL(KContextServerName);
}

CSharableSession* CContextServer::NewSessionL(const TVersion& aVersion) const
{
	// check version
	if (!User::QueryVersionSupported(TVersion(KContextServMajorVersionNumber,
                                                KContextServMinorVersionNumber,
                                                KContextServBuildVersionNumber),
                                                aVersion))
	{
		User::Leave(KErrNotSupported);
	}
	
	// create new session
	RThread client = Message().Client();
	return CContextServerSession::NewL(client, *const_cast<CContextServer*> (this));
}


void CContextServer::IncrementSessions()
{
    	iSessionCount++;
}

void CContextServer::DecrementSessions()
{
    	iSessionCount--;
    	if (iSessionCount <= 0)
	{
      		iSessionCount =0;
			iJabberClient->Disconnect();
		CActiveScheduler::Stop();
	}    
}

TInt CContextServer::CheckedRunError(TInt aError)
{
	if (aError == KErrBadDescriptor)
	{
        	// A bad descriptor error implies a badly programmed client, so panic it;
        	// otherwise report the error to the client
        	PanicClient(Message(), EBadDescriptor);
	}
	else
	{
		Message().Complete(aError);
	}

	//
	// The leave will result in an early return from CServer::RunL(), skipping
	// the call to request another message. So do that now in order to keep the
	// server running.
	ReStart();

	return KErrNone;	// handled the error fully
}

void CContextServer::PanicClient(const RMessage& aMessage, TContextServPanic aPanic)
{
	aMessage.Panic(KContextServer, aPanic);
}

void CContextServer::PanicServer(TContextServPanic aPanic)
{
    	User::Panic(KContextServer, aPanic);
}

void CContextServer::RunL()
{
	TRAPD(err, CServer::RunL());
	if (err!=KErrNone) {
		TBuf<20> msg;
		msg.Format(_L("Error in RunL: %d"), err);
		Log(msg);
		CheckedRunError(err);
	}
}

void CContextServer::ThreadFunctionL()
{
	#if defined (__WINS__)
	UserSvr::ServerStarted();
	#endif

	__UHEAP_MARK;

    	// Construct active scheduler
    	CActiveScheduler* activeScheduler = new (ELeave) CBaActiveScheduler;

    	CleanupStack::PushL(activeScheduler) ;

    	// Install active scheduler
    	// We don't need to check whether an active scheduler is already installed
    	// as this is a new thread, so there won't be one
    	CActiveScheduler::Install(activeScheduler);

    	// Construct our server
	CContextServer::NewLC();    

	RSemaphore semaphore;
	if (semaphore.CreateGlobal(KContextServerSemaphoreName, 0)!=KErrNone) {
		User::LeaveIfError(semaphore.OpenGlobal(KContextServerSemaphoreName));
	}
	
	// Semaphore opened ok
	semaphore.Signal();
	semaphore.Close();

	// Start handling requests
	CActiveScheduler::Start();
		 
	CleanupStack::PopAndDestroy(2,activeScheduler); // CServer

	__UHEAP_MARKEND;

}

TInt CContextServer::ThreadFunction(TAny* /*aNone*/)
{
    	CTrapCleanup* cleanupStack = CTrapCleanup::New();
	if (cleanupStack == NULL)
	{
      	PanicServer(ECreateTrapCleanup);
	}

	TRAPD(err, ThreadFunctionL());
    	if (err != KErrNone)
	{
        	PanicServer(ESrvCreateServer);
	}
	
    	delete cleanupStack;
    	cleanupStack = NULL;

	return KErrNone;
}


//---------------------------------------------------------------------------

void CContextServer::TerminateContextServer()
{
	iJabberClient->Disconnect();

	if ( iMessageFromBook!=RMessage() ) 
	{
		iMessageFromBook.Complete(EContextServerTerminated);
		
	} 

	if ( iMessageFromLog!=RMessage() )
	{
		iMessageFromLog.Complete(EContextServerTerminated);	
	}
	//Log();
	CActiveScheduler::Stop();
}

void CContextServer::ConnectToPresenceServer(TDesC & username, TDesC & password, TDesC & server, TUint32 accessPoint)
{
	iAccessPoint = accessPoint;
	iUsername.Copy(username);
	iPassword.Copy(password);
	iServer.Copy(server);

	iJabberClient->ConnectL(username,password,server,accessPoint);
}

void CContextServer::ConnectToPresenceServer(const RMessage &aMessage)
{
	// local copy
	iMessageFromLog = aMessage;

	const TAny* clientsDescriptor0 = iMessageFromLog.Ptr0();
	TBuf<30> username;
	iMessageFromLog.ReadL(clientsDescriptor0, username);

	const TAny* clientsDescriptor1 = iMessageFromLog.Ptr1();
	TBuf<30> password;
	iMessageFromLog.ReadL(clientsDescriptor1, password);

	const TAny* clientsDescriptor2 = iMessageFromLog.Ptr2();
	TBuf<30> server;
	iMessageFromLog.ReadL(clientsDescriptor2, server);

	const TAny* clientsDescriptor3 = iMessageFromLog.Ptr3();
	TUint32 accessPointId;
	TPckg<TUint32> apPackage(accessPointId);
	iMessageFromLog.ReadL(clientsDescriptor3, apPackage,0);

	ConnectToPresenceServer(username,password,server,accessPointId);
}

void CContextServer::HandleRequestPresenceNotification(const RMessage &aMessage)
{
	TPresenceInfo info = iPresenceInfo->GetNewPresenceInfo();
	
	if (info.newInfo == 1)
	{
		TInt len0 = aMessage.Client().GetDesMaxLength(aMessage.Ptr0());
		TInt len1 = aMessage.Client().GetDesMaxLength(aMessage.Ptr1());

		if ( len0 < info.contact.Length() || len1 < info.info.Length())
		{
			iPresenceInfo->SetAsNew(info.contact);
			
			TPckgC<TInt> len0Package(info.info.Length());
			aMessage.WriteL(aMessage.Ptr0(),len0Package,0);

			TPckgC<TInt> len1Package(info.contact.Length());
			aMessage.WriteL(aMessage.Ptr1(),len1Package,0);
			aMessage.Complete(EBufferTooSmall);
		}
		else
		{
			TPtr8 ptr(reinterpret_cast<TUint8*>(&info.send_timestamp), sizeof(info.send_timestamp), sizeof(info.send_timestamp));

			//DEBUG
			TBuf<10> buf;
			info.send_timestamp.FormatL(buf, _L("%H%T%S"));
			RDebug::Print(buf);
			
			
			aMessage.WriteL(aMessage.Ptr0(),info.contact,0);
			aMessage.WriteL(aMessage.Ptr1(),info.info,0);
			aMessage.WriteL(aMessage.Ptr2(),ptr,0);
			aMessage.Complete(ERequestCompleted);
		}	
	}
	else
	{
		iMessageFromBook = aMessage;
	}	
}

void CContextServer::SuspendConnection(const RMessage &aMessage)
{
	iJabberClient->Disconnect();
	aMessage.Complete(ERequestCompleted);
	if (iMessageFromBook != RMessage())
	{
		iMessageFromBook.Complete(EServerUnreachable);
		iMessageFromBook = RMessage();
	}
}

void CContextServer::ResumeConnection(const RMessage &aMessage)
{
	iMessageFromLog = aMessage;
	//iConnectionOpener->MakeConnectionL(iAccessPoint);
	iJabberClient->ConnectL(iUsername,iPassword,iServer,iAccessPoint);
}

CPresenceInfo *  CContextServer::GetPresenceInfo()
{
	return iPresenceInfo;
}

CSocketsEngine * CContextServer::GetJabberClient()
{
	return iJabberClient;
}


// from MConnectionOpenerObserver --------------------------------------------------------------------
/*void CContextServer::success(CBase* source)
{
	iJabberClient->ConnectL(iUsername,iPassword,iServer);
}

void CContextServer::error(CBase* source, TInt code, const TDesC& reason)
{
	Log(reason);
}

void CContextServer::info(CBase* source, const TDesC& msg)
{
	Log(msg);
}*/

// from MContextServerNotifier --------------------------------------------------------------------
void CContextServer::NotifyConnected()
{
	if (iMessageFromLog != RMessage() )
	{
		iMessageFromLog.Complete(ERequestCompleted);
		iMessageFromLog = RMessage();
	}
	
}

void CContextServer::NotifyNewPresenceInfo(const TDesC & from, const TDesC & info, 
					   const TTime& send_timestamp)
{
	RDebug::Print(from.Left(128));
	RDebug::Print(info.Left(128));

	//DEBUG
	TBuf<10> buf;
	send_timestamp.FormatL(buf, _L("%H%T%S"));
	RDebug::Print(buf);
	
	iPresenceInfo->Update(from,info,send_timestamp);
	
	if (iMessageFromBook != RMessage()) 
	{
		TPresenceInfo info = iPresenceInfo->GetNewPresenceInfo();
		
		TInt len0 = iMessageFromBook.Client().GetDesMaxLength(iMessageFromBook.Ptr0());
		TInt len1 = iMessageFromBook.Client().GetDesMaxLength(iMessageFromBook.Ptr1());

		if ( len0 < info.contact.Length() || len1 < info.info.Length() )
		{
			iPresenceInfo->SetAsNew(info.contact);
			
			TPckgC<TInt> len0Package(info.contact.Length());
			iMessageFromBook.WriteL(iMessageFromBook.Ptr0(),len0Package,0);
			
			TPckgC<TInt> len1Package(info.info.Length());
			iMessageFromBook.WriteL(iMessageFromBook.Ptr1(),len1Package,0);
			iMessageFromBook.Complete(EBufferTooSmall);
		}
		else
		{
			iMessageFromBook.WriteL(iMessageFromBook.Ptr0(),info.contact,0);
			iMessageFromBook.WriteL(iMessageFromBook.Ptr1(),info.info,0);
			
			TPckgC<TTime> package(info.send_timestamp);
			iMessageFromBook.WriteL(iMessageFromBook.Ptr2(),package,0);

			iMessageFromBook.Complete(ERequestCompleted);
		}	
		iMessageFromBook = RMessage();
	}
}

void CContextServer::ReportError(TContextServRqstComplete aErrorType, TDesC & aErrorCode, TDesC & aErrorValue)
{
	if (iMessageFromBook != RMessage())
	{
		iMessageFromBook.Complete(aErrorType);
		iMessageFromBook = RMessage();
	}

	if (iMessageFromLog != RMessage())
	{
		iMessageFromLog.WriteL(iMessageFromLog.Ptr0(),aErrorCode,0);
		iMessageFromLog.WriteL(iMessageFromLog.Ptr1(),aErrorValue,0);
		iMessageFromLog.Complete(aErrorType);
		iMessageFromLog = RMessage();
	}
	
}

void CContextServer::CancelRequest(RMessage Msg)
{
	if (Msg.Client().Id()==iMessageFromBook.Client().Id()) {
		iMessageFromBook.Complete(KErrCancel);
		iMessageFromBook=RMessage();
	} else 	if (Msg.Client().Id()==iMessageFromLog.Client().Id()) {
		iMessageFromLog.Complete(KErrCancel);
		iMessageFromLog=RMessage();
	}
}
