#if !defined(TRANSFER_H_INCLUDED)

#define TRANSFER_H_INCLUDED

#include <e32base.h>
#include <f32file.h>
#include <e32std.h>
#include <sendui.h>
#include <AknQueryDialog.h>

#include "file_output_base.h"
#include "status_notif.h"
#include "ftp.h"
#include "http.h"
#include "app_context.h"
#include "log_comm.h"

class CTransferBase: public MContextBase, public CBase
{
public:
	virtual ~CTransferBase();
	
	void add_filesL(const TDesC& file, bool leave_last);
protected:
	bool GetFiles(TTime AfterTime=TTime(0));

	CDesC16ArrayFlat* file_names;

	CTransferBase(MApp_context& Context);
	void ConstructL(i_status_notif* callback, const TDesC& dir1, const TDesC& dir2);
	CDesC16ArrayFlat* send_dirs;
	CArrayFixFlat<bool>* send_leave;
	TFileName dir_prefix1, dir_prefix2;
	i_status_notif* cb;
};

class CSendUITransfer: public CTransferBase {
public:
	static CSendUITransfer* NewL(MApp_context& Context, 
		i_status_notif* callback, TInt cmdid, const TDesC& dir1, const TDesC& dir2);
	void DisplayMenuL(CEikMenuPane& aMenuPane);
	void DisplaySendMenuL(CEikMenuPane& aMenuPane, TInt pos);
	bool transfer_files(TInt cmdid); // true when starting to write, false if already writing
	~CSendUITransfer();
private:
	bool in_progress;
	CSendUITransfer(MApp_context& Context);
	void ConstructL(i_status_notif* callback, TInt cmdid, const TDesC& dir1, const TDesC& dir2);
	CSendAppUi* sendui;

};

class CFtpTransfer: public CTransferBase, public MSocketObserver
{
public:
	enum TConnType { WAP, FTP, HTTP };
	static CFtpTransfer* NewL(MApp_context& Context, 
		i_status_notif* callback, const TDesC& dir1, const TDesC& dir2,
		TConnType ConnType);

	void TransferFiles(const TDesC& Meta=KNullDesC, TTime AfterTime=TTime(0));
	~CFtpTransfer();
private:
	void ConstructL(i_status_notif* callback, const TDesC& dir1, const TDesC& dir2,
		TConnType ConnType);
	CFtpTransfer(MApp_context& Context);

	virtual void success(CBase* source);
	virtual void error(CBase* source, TInt code, const TDesC& reason);
	virtual void info(CBase* source, const TDesC& msg);
	bool GetImsi(TDes& aImsi);

	enum state { IDLE, CONNECTING, MKDING, CWDING, SENDING_FILE, CLOSING };
	state current_state;

	int	file_index;
	CHttp*	http;
	TConnType	iConnType;
	MAdvGsmPhoneInformation::TSubscriberId iImsi;
	TBuf<200>	iMeta;
};

class CPeriodicTransfer: public MContextBase, public i_status_notif, public MTimeOut, public CBase
{
public:
	static CPeriodicTransfer* NewL(MApp_context& Context, int hours, MSocketObserver* callback);
	~CPeriodicTransfer();
	void Transfer(bool MakeCommLog=true);
private:
	CPeriodicTransfer(MApp_context& Context, int hours, MSocketObserver* callback);
	void ConstructL();

	enum state { IDLE, GETTING_COMM_LOG, FTPING };
	state current_state;
	void expired();
	virtual void finished();
	virtual void error(const TDesC& descr);
	virtual void status_change(const TDesC& status);

	CTimeOut	*iTimer;
	int		iHours, iCount;
	MSocketObserver	*iCb;
	CFtpTransfer	*iFtp;
	Clog_comm*	iCommLog;
	TTime		iCommLogFrom;
};


#endif
