#include "discover.h"

#define POLL_INTERVAL 5*60

Cdiscover::Cdiscover(MApp_context& Context) : CCheckedActive(CCheckedActive::EPriorityIdle, _L("Cdiscover")), 
Mlog_base_impl(Context),
current_state(IDLE)
{
	CALLSTACKITEM(_L("Cdiscover::Cdiscover"));

}

void Cdiscover::ConstructL() 
{
	CALLSTACKITEM(_L("Cdiscover::ConstructL"));

	Mlog_base_impl::ConstructL(_L("devices"), _L(""));

#ifndef __WINS__
	CActiveScheduler::Add(this); // add to scheduler
	User::LeaveIfError(timer.CreateLocal());

	User::LeaveIfError(socket_server.Connect());
	TProtocolName aName=_L("BTLinkManager");
	User::LeaveIfError(socket_server.FindProtocol(aName, pInfo));
	User::LeaveIfError(hr.Open(socket_server,pInfo.iAddrFamily,pInfo.iProtocol));

	TTimeIntervalMicroSeconds32 w(POLL_INTERVAL*1000*1000);
	timer.After(iStatus, w);
	SetActive();
#endif
}

Cdiscover* Cdiscover::NewL(MApp_context& Context)
{
	CALLSTACKITEM(_L("Cdiscover::NewL"));

	Cdiscover* ret=new (ELeave) Cdiscover(Context);
	CleanupStack::PushL(ret);
	ret->ConstructL();
	CleanupStack::Pop();
	return ret;
}

void Cdiscover::release()
{	
	CALLSTACKITEM(_L("Cdiscover::release"));

	timer.Close();
	hr.Close();
	socket_server.Close();
}

Cdiscover::~Cdiscover()
{
	CALLSTACKITEM(_L("Cdiscover::~Cdiscover"));

	Cancel();
	release();
}	

void Cdiscover::DoCancel()
{
	CALLSTACKITEM(_L("Cdiscover::DoCancel"));

	switch(current_state) {
	case IDLE:
		timer.Cancel();
		break;
	default:
		hr.Cancel();
		break;
	}
}

const TDesC& Cdiscover::get_value()
{
	CALLSTACKITEM(_L("Cdiscover::get_value"));

	return names;
}


void Cdiscover::CheckedRunL()
{
	CALLSTACKITEM(_L("Cdiscover::CheckedRunL"));

	TBuf<30> msg;
	if (iStatus.Int()!=KErrNone && iStatus.Int()!=KErrHostResNoMoreResults ) {
		_LIT(stat_err, "error: %d");
		msg.Format(stat_err, iStatus.Int());
		post_new_value(msg, Mlogger::ERR);
		TTimeIntervalMicroSeconds32 w(POLL_INTERVAL*1000*1000);
		timer.After(iStatus, w);
		SetActive();
		current_state=IDLE;
		return;
	}
	switch(current_state) {
	case FIRST_SEARCH:
		if (iStatus.Int()!=KErrHostResNoMoreResults) {
			names.Zero();
			TBuf<30> addr;
			TBTSockAddr btaddr(name().iAddr);
			btaddr.BTAddr().GetReadable(addr);
			names.Append(addr);
			names.Append(_L(" ["));
			names.Append(name().iName);
			names.Append(_L("] "));
			hr.Next(name, iStatus);
			SetActive();
			current_state=NEXT_SEARCH;
		} else {
			if (prev_env.Length()>0) {
				post_new_value(_L(""));
				prev_env.Zero();
			}
			current_state=IDLE;
			TTimeIntervalMicroSeconds32 w(POLL_INTERVAL*1000*1000);
			timer.After(iStatus, w);
			current_state=IDLE;
			SetActive();
		}

		break;
	case NEXT_SEARCH:
		if (iStatus.Int()!=KErrHostResNoMoreResults) {
			if (! (names.Length()+32+name().iName.Length() > names.MaxLength()) ) {
				TBuf<30> addr;
				TBTSockAddr btaddr(name().iAddr);
				btaddr.BTAddr().GetReadable(addr);
				names.Append(addr);
				names.Append(_L(" ["));
				names.Append(name().iName);
				names.Append(_L("] "));
			}
			hr.Next(name, iStatus);
			SetActive();
		} else {
			if (prev_env.Compare(names)) {
				post_new_value(names);
				prev_env=names;
			}
			current_state=IDLE;
			TTimeIntervalMicroSeconds32 w(POLL_INTERVAL*1000*1000);
			timer.After(iStatus, w);
			current_state=IDLE;
			SetActive();
		}
		break;
	case IDLE:
		addr.SetIAC(KGIAC);
		addr.SetAction(KHostResInquiry|KHostResName|KHostResIgnoreCache);
		hr.GetByAddress(addr, name, iStatus);

		SetActive();
		
		current_state=FIRST_SEARCH;
		break;
	}
}
