#include "presence_publisher.h"
#include "symbian_auto_ptr.h"
#include "cl_settings.h"
#include "i_logger.h"
#include "cellmap.h"
#include "eikenv.h"
#include "status_notif.h"


#pragma warning(disable: 4706) // assignment withing conditional expression

_LIT(KPresencePublisher, "PresencePublisher");

CPresencePublisher* CPresencePublisher::NewL(MApp_context& Context, i_status_notif* CallBack, MPresencePublisherListener& aListener)
{
	CALLSTACKITEM(_L("CPresencePublisher::NewL"));

	auto_ptr<CPresencePublisher> ret( new (ELeave) CPresencePublisher(Context, CallBack, aListener) );
	ret->ConstructL();
	return ret.release();
}

CPresencePublisher::~CPresencePublisher()
{
	CALLSTACKITEM(_L("CPresencePublisher::~CPresencePublisher"));

	Settings().CancelNotifyOnChange(SETTING_PRESENCE_ENABLE, this);
	Cancel();
	delete iWait;
	iSession.Close();
	delete iSendPresence;
	delete iPresence;
}

void CPresencePublisher::SuspendConnection()
{
	CALLSTACKITEM(_L("CPresencePublisher::SuspendConnection"));

	if (iNextOp==EDisable) return;
	iWait->Cancel();

	switch (iCurrentState) {
	case EDisabling:
	case EDisabled:
		break;
	default:
		if (IsActive()) {
			iNextOp=ESuspend;
		} else {
			iSession.MsgSuspendConnection(iStatus);
			SetActive();
			iCurrentState=ESuspending;
		}
		break;
	}
}

bool CPresencePublisher::Disabled()
{
	CALLSTACKITEM(_L("CPresencePublisher::Disabled"));

	if (iCurrentState==EDisabling || iCurrentState==EDisabled || iNextOp==EDisable) {
		return true;
	}
	return false;
}

void CPresencePublisher::ResumeConnection()
{
	CALLSTACKITEM(_L("CPresencePublisher::ResumeConnection"));

	if (iNextOp==EDisable) return;
	iWait->Cancel();

	switch (iCurrentState) {
	case EDisabling:
	case EDisabled:
		break;
	default:
		if (IsActive()) {
			iNextOp=EResume;
		} else {
			iSession.MsgResumeConnection(iStatus);
			iCurrentState=EResuming;
			SetActive();
		}
		break;
	}
}

bool CPresencePublisher::ConnectionSuspended()
{
	CALLSTACKITEM(_L("CPresencePublisher::ConnectionSuspended"));

	if (iCurrentState==ESuspending || iCurrentState==ESuspended || iNextOp==ESuspend) return true;
	return false;
}

CPresencePublisher::CPresencePublisher(MApp_context& Context, i_status_notif* CallBack , MPresencePublisherListener& aListener): 
CCheckedActive(EPriorityIdle, _L("CPresencePublisher")), MContextBase(Context), iCallBack(CallBack), iListener(aListener)

{
	CALLSTACKITEM(_L("CPresencePublisher::CPresencePublisher"));

}

void CPresencePublisher::ConstructL()
{
	CALLSTACKITEM(_L("CPresencePublisher::ConstructL"));

	iWait=CTimeOut::NewL(*this);
	iPresence=CXmlBuf::NewL(256);

	iCurrentState=EDisabled;
	Settings().NotifyOnChange(SETTING_PRESENCE_ENABLE, this);

	TInt enabled=1;
	if (! Settings().GetSettingL(SETTING_PRESENCE_ENABLE, enabled) ) {
		enabled=1;
	}

	CActiveScheduler::Add(this);
	Settings().GetSettingL(SETTING_IP_AP, iAP);
	if (enabled) {
		iWaitTime=1;
		Restart();
	} else {
		iCurrentState=EDisabled;
	}
}

void CPresencePublisher::SettingChanged(TInt Setting)
{
	CALLSTACKITEM(_L("CPresencePublisher::SettingChanged"));

	if (Setting!=SETTING_PRESENCE_ENABLE) return;

	TInt enabled=1;
	if (! Settings().GetSettingL(SETTING_PRESENCE_ENABLE, enabled) ) {
		enabled=1;
	}
	if (enabled) {
		switch(iCurrentState) {
		case EDisabled:
			iWaitTime=1;
			Restart();
		default:
			break;
		}
	} else {
		if (iCurrentState==EDisabled || iCurrentState==EDisabling) return;
		iWait->Cancel();
		if (iCurrentState==EReconnecting || iCurrentState==ESuspended) {
			Cancel();
			iCurrentState=EDisabled;
			return;
		}
		if (iCurrentState!=ESuspending) {
			Cancel();
			iSession.MsgSuspendConnection(iStatus);
			iCurrentState=ESuspending;
			SetActive();
		}
		iNextOp=EDisable;
	}
}

// Mlogger
void CPresencePublisher::register_source(const TDesC& name, const TDesC& initial_value, const TTime& time)
{
	CALLSTACKITEM(_L("CPresencePublisher::register_source"));

	if (initial_value.Length()>0) {
		new_value(VALUE, name, initial_value, time);
	}
}

void CPresencePublisher::new_value(log_priority priority, const TDesC& name, const TDesC& value, const TTime& time)
{
	CALLSTACKITEM(_L("CPresencePublisher::new_value"));

	TBuf<20> n=name;

	if (priority!=VALUE) return;

	if (! name.Compare(KLog_cellid) ) {
		CCellMap::Parse(value, iCell.iCellId, iCell.iLAC, iCell.iNetwork);
		iCell.iUpdated=time;
	} else if (! name.Compare(KLog_profile) ) {
		TLex16 lex(value);
		lex.Val(iProfile.iProfileId);
		lex.SkipCharacters();
		lex.SkipSpace(); lex.Mark();
		lex.SkipCharacters();
		iProfile.iProfileName=lex.MarkedToken();
		lex.SkipSpace(); lex.Inc(1); lex.SkipSpace();
		lex.Val(iProfile.iRingingType);
		lex.SkipCharacters();
		lex.SkipSpace();
		lex.Val(iProfile.iRingingVolume);
		lex.SkipCharacters();
		lex.SkipSpace(); lex.Mark();
		lex.SkipCharacters();
		TBuf<10> tmp=lex.MarkedToken();
		if (! tmp.Left(2).Compare( _L("On") ) ) {
			iProfile.iVibra=ETrue;
		} else {
			iProfile.iVibra=EFalse;
		}

		iProfile.iUpdated=time;
	} else if (! name.Compare(KLog_activity) ) {
		if ( ! value.Compare(_L("idle")) ) {
			iActivity.iMode=TActivityInfo::EIdle;
		} else {
			iActivity.iMode=TActivityInfo::EActive;
		}
		iActivity.iUpdated=time;
	} else if (! name.Compare(KLog_cellname) ) {
		TInt fieldsep;
		TInt beg=0;
		TInt valuesep;
		fieldsep=value.Locate(';');
		if (fieldsep==KErrNotFound) fieldsep=value.Length();

		TInt iPrevFlags=iLocation.iFlags;
		iLocation.iUpdated=time;
		iLocation.iFlags=0;
		while( fieldsep!=KErrNotFound ) {
			valuesep=value.Mid(beg, fieldsep-beg).Locate(':');
			if (valuesep==KErrNotFound) valuesep=beg;
			else valuesep+=2;
			TPtrC16 field=value.Mid(beg, valuesep-beg);
			TPtrC16 fieldval=value.Mid(beg+valuesep, fieldsep-beg-valuesep);

			if (! field.Left(4).Compare(_L("last")) ) {
				if (! (iPrevFlags & TLocInfo::EPrevious) || 
						iLocation.iCurrentOrPrevious->Des().Compare(fieldval) ) {
					delete iLocation.iCurrentOrPrevious; iLocation.iCurrentOrPrevious=0;
					iLocation.iCurrentOrPrevious=fieldval.AllocL();
					iLocation.iCoPUpdated=time;
				}
				iLocation.iFlags|=TLocInfo::EPrevious;
			} else if (! field.Left(4).Compare(_L("next")) ) {
				if (! (iPrevFlags & TLocInfo::ENext) || 
						iLocation.iNext->Des().Compare(fieldval) ) {
					delete iLocation.iNext; iLocation.iNext=0;
					iLocation.iNext=fieldval.AllocL();
					iLocation.iNUpdated=time;
				}
				iLocation.iFlags|=TLocInfo::ENext;
			} else {
				if (! (iPrevFlags & TLocInfo::ECurrent) || 
						iLocation.iCurrentOrPrevious->Des().Compare(fieldval) ) {
					delete iLocation.iCurrentOrPrevious; iLocation.iCurrentOrPrevious=0;
					iLocation.iCoPUpdated=time;
					iLocation.iCurrentOrPrevious=fieldval.AllocL();
				}
				iLocation.iFlags|=TLocInfo::ECurrent;
			}
			beg=fieldsep;
			fieldsep=value.Mid(beg).Locate(';');
		}
	}
	MakePresence();
	if (iCurrentState==EConnected) {
		SendUpdate();
	} else {
		iNewValue=true;
	}
}

void CPresencePublisher::MakePresence()
{
	CALLSTACKITEM(_L("CPresencePublisher::MakePresence"));

	iPresence->Zero();
	iPresence->BeginElement(_L("presence"));

	MXmlInfo* events[] = { &iCell, &iLocation, &iActivity, &iProfile, 0 };

	MXmlInfo* event;
	TBuf<20> buffer;
	for (int i=0; event=events[i]; i++) {

		iPresence->BeginElement(_L("event"));
		_LIT(KFormatTxt,"%04d%02d%02dT%02d%02d%02d");
		TDateTime dt=event->iUpdated.DateTime();

		buffer.Format(KFormatTxt, dt.Year(), (TInt)dt.Month()+1, (TInt)dt.Day()+1,
			dt.Hour(), dt.Minute(), dt.Second());
		iPresence->Leaf(_L("datetime"), buffer);
		event->Persist(iPresence);
		iPresence->EndElement(_L("event"));
	}
	iPresence->EndElement(_L("presence"));
	//RDebug::Print(iPresence->Buf());

}

void CPresencePublisher::SendUpdate()
{
	CALLSTACKITEM(_L("CPresencePublisher::SendUpdate"));

	delete iSendPresence; iSendPresence=0;
	iSendPresence=iPresence->Buf().AllocL();

	iListener.NotifyNewPresence(*iSendPresence);

	iSession.MsgUpdateUserPresence(*iSendPresence, iStatus);

	SetActive();
	iCurrentState=ESendingUpdate;
	iNewValue=false;

	//User::Leave(1003);

}

void CPresencePublisher::unregister_source(const TDesC& /*name*/, const TTime& /*time*/)
{
	CALLSTACKITEM(_L("CPresencePublisher::unregister_source"));

}

const TDesC& CPresencePublisher::name() const
{
	CALLSTACKITEM(_L("CPresencePublisher::name"));

	return KPresencePublisher;
}

void CPresencePublisher::Restart()
{
	CALLSTACKITEM(_L("CPresencePublisher::Restart"));

	Cancel();
	iSession.Close();
	iStatus=EReconnecting;
	iWait->Wait(iWaitTime);
}

void CPresencePublisher::CheckedRunL()
{
	CALLSTACKITEM(_L("CPresencePublisher::CheckedRunL"));

	TBuf<50> msg;
	if (iStatus<0) {
		iWaitTime=(int)(iWaitTime*1.5);

		// depending on the error, we should not always restart
		//Restart();
		if (iStatus == -1 /*EIdentificationError*/) 
		{
			TBuf<100> error;
			error.Append(_L("ID Error: "));
			error.Append(iUser);
			
			iCallBack->error(error);
			CEikonEnv::Static()->AlertWin(error,iPass);
		}
		else 
		{
			msg.Format(_L("Presence error %d, restarting"), iStatus.Int());
			iCallBack->status_change(msg);
			Restart();
		}
		return;
	}

	iWaitTime=10;

	switch (iCurrentState) {
	case EConnecting:
	case EResuming:
		msg=_L("Presence Connected");
	case ESendingUpdate:
		iCurrentState=EConnected;
		break;
	case ESuspending:
		msg=_L("Presence Suspended");
		iCurrentState=ESuspended;
		break;
	default:
		msg=_L("Presence Restarting");
		Restart();
		break;
	}

	if (msg.Length()>0) iCallBack->status_change(msg);

	switch (iNextOp) {
	case ESuspend:
		iSession.MsgSuspendConnection(iStatus);
		iCurrentState=ESuspending;
		break;
	case EResume:
		iSession.MsgResumeConnection(iStatus);
		iCurrentState=EResuming;
		break;
	case EDisable:
		iSession.Close();
		iCurrentState=EDisabled;
		break;
	};
	iNextOp=ENone;

	if (iNewValue && iCurrentState==EConnected) {
		SendUpdate();
	}
}
		
void CPresencePublisher::DoCancel()
{
	CALLSTACKITEM(_L("CPresencePublisher::DoCancel"));

	iSession.Cancel();
}

TInt CPresencePublisher::CheckedRunError(TInt /*aError*/)
{
	CALLSTACKITEM(_L("CPresencePublisher::CheckedRunError"));

	Restart();
	
	return KErrNone;
}

void CPresencePublisher::expired()
{
	CALLSTACKITEM(_L("CPresencePublisher::expired"));

	TBuf<50> tmp;
	if (Settings().GetSettingL(SETTING_JABBER_NICK, tmp) && Settings().GetSettingL(SETTING_JABBER_PASS, iPass)) {
		TInt sep=tmp.Locate('@');
		if (sep!=KErrNotFound) {
			TInt ret=iSession.ConnectToContextServer();
			if (ret!=KErrNone) {
				Restart();
				return;
			}
			iUser=tmp.Mid(0, sep);
			iServer=tmp.Mid(sep+1);

			iCallBack->status_change(_L("Presence Connecting"));
			iSession.MsgConnectToPresenceServer(iUser, iPass,
				iServer, iAP, iStatus);
			iCurrentState=EConnecting;
			SetActive();
		}
	}
}

