#include "cb_presence.h"
#include "symbian_auto_ptr.h"
#include "contextCommon.h"

const TInt CPresenceHolder::KDefaultBufferSize = 256;

CPresenceHolder* CPresenceHolder::NewL(CJabberData& JabberData)
{
	auto_ptr<CPresenceHolder> ret(new (ELeave) CPresenceHolder(JabberData));
	ret->ConstructL();
	return ret.release();
}

CPresenceHolder::~CPresenceHolder()
{
	Cancel();
	if (iSessionOpen) iSession.Close();
	delete iWait;
	delete iListeners;
	delete iPresenceData;

	delete iContact;
	delete iPresence;
	
}

void CPresenceHolder::AddListener(MPresenceListener* Listener)
{
	iListeners->AppendL(Listener);
}

const CPresenceData* const CPresenceHolder::GetPresence(TInt ContactId) const
{
	return (CPresenceData*) iPresenceData->GetData(ContactId);
}

CPresenceHolder::CPresenceHolder(CJabberData& JabberData) : iJabberData(JabberData),
	CCheckedActive(EPriorityStandard, _L("CPresenceHolder")),iP(0,0),iC(0,0)
{
    	CActiveScheduler::Add(this);
}

void CPresenceHolder::Start()
{
	if (iSession.ConnectToContextServer() == KErrNone)
	{
		iSessionOpen=true;
		RequestPresence();
	} else {
		Restart();
	}
}

void CPresenceHolder::Restart()
{
	Cancel();
	iWait->Cancel();
	if (iSessionOpen) {
		iSession.Close();
	}
	iSessionOpen=false;
	RDebug::Print(_L("restarting"));
	iWait->Wait(20);
}

void CPresenceHolder::expired()
{
	Start();
}

void CPresenceHolder::ConstructL()
{
	iWait=CTimeOut::NewL(*this);

	iPresenceData=CGenericIntMap::NewL();
	iPresenceData->SetDeletor(CPresenceDataDeletor);
	iListeners=CList<MPresenceListener*>::NewL();

	iContact = HBufC::NewL(KDefaultBufferSize/4);
	iPresence= HBufC::NewL(KDefaultBufferSize);

	iSendTimeStamp = TTime();

	Start();
}

void CPresenceHolder::NewPresence(const TDesC& Nick, const TDesC& Info, const TTime & send_timestamp)
{
	TInt contact;
	contact=iJabberData.GetContactIdL(Nick);
	if (contact==KErrNotFound) return;

	auto_ptr<CPresenceData> data( CPresenceData::NewL(Info, send_timestamp) );
	iPresenceData->AddDataL(contact, data.get(), true);
	CPresenceData* dp=data.release();

	CList<MPresenceListener*>::Node* i=iListeners->iFirst;
	while (i) {
		TRAPD(err, i->Item->PresenceChangedL(contact, *dp));
		i->Item->Notify(_L(""));
		i=i->Next;
	}
}

CPresenceData* CPresenceHolder::GetPresence(TInt ContactId)
{
	return (CPresenceData*)iPresenceData->GetData(ContactId);
}

void CPresenceHolder::CPresenceDataDeletor(void* p)
{
	CPresenceData* b=(CPresenceData*)p;
	delete b;
}

void CPresenceHolder::CheckedRunL()
{
	if (iStatus == ERequestCompleted)
	{
		switch (current_state)
		{
			case EWaitingForPresence:
				{//DEBUG
				TBuf<10> buf;
				iSendTimeStamp.FormatL(buf, _L("%H%T%S")),
					RDebug::Print (buf);
				


				NewPresence(iC,iP, iSendTimeStamp);
				//RDebug::Print(_L("new presence:"));
				//RDebug::Print(iC);
				//RDebug::Print(iP.Left(128));

				RequestPresenceNotification();
				break;
				}
			default:
				Restart();
				// ASSERT(0); //Unexpected error
				break;
		}
	}
	else if (iStatus == EBufferTooSmall)
	{
		iContact = iContact->ReAllocL(iC.MaxLength() *2);
		iC.Set(iContact->Des());

		iPresence = iPresence->ReAllocL(iP.MaxLength() *2);
		iP.Set(iPresence->Des());

		RequestPresenceNotification();
	}
	else if (iStatus == EServerUnreachable || iStatus == EIdentificationError )
	{
		CList<MPresenceListener*>::Node* i=iListeners->iFirst;
		while (i) 
		{
			i->Item->Notify(_L("Server unreachable"));
			i=i->Next;
		}
		RequestPresenceNotification();
		//Restart();
	}
	else if (iStatus == EContextServerTerminated)
	{
		CList<MPresenceListener*>::Node* i=iListeners->iFirst;
		while (i) 
		{
			i->Item->Notify(_L("Server Stopped"));
			i=i->Next;
		}
		// do nothing, it's a proper stop of the context server ...
	}
	else 
	{
		Restart();
	}
}

void CPresenceHolder::RequestPresence()
{
	if(!IsActive())
	{
		current_state = EWaitingForPresence;
		
		iC.Set(iContact->Des());
		iP.Set(iPresence->Des());

		iSession.MsgRequestPresenceInfo(iC, iP, iSendTimeStamp, iStatus);
		SetActive();
	}	
}

void CPresenceHolder::RequestPresenceNotification()
{
	if(!IsActive())
	{
		current_state = EWaitingForPresence;

		iC.Set(iContact->Des());
		iP.Set(iPresence->Des());

		iSession.MsgRequestPresenceNotification(iC, iP, iSendTimeStamp, iStatus);
		SetActive();
	}		
}

void CPresenceHolder::CancelRequest()
{
    	Cancel() ;
}

void CPresenceHolder::DoCancel()
{
	iSession.Cancel();
}

